package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.representation.Form;
import com.sun.jersey.oauth.client.OAuthClientFilter;
import com.sun.jersey.oauth.signature.HMAC_SHA1;
import com.sun.jersey.oauth.signature.OAuthParameters;
import com.sun.jersey.oauth.signature.OAuthSecrets;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.List;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

/**
 *
 * @author joeri
 */
public class DaliCoreExternalNetwork extends ExternalNetwork {
	private String hostname;

  public DaliCoreExternalNetwork(String hostname, String consumerKey, String consumerSecret) {
    super(consumerKey, consumerSecret, "dalicore");

		this.hostname = hostname;
  }

  public DaliCoreExternalNetwork(String hostname, String consumerKey, String consumerSecret, String name) {
    super(consumerKey, consumerSecret, name);

		this.hostname = hostname;
  }

  @Override
  public Response connect(String callback) {
    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").callback(callback);
    OAuthSecrets secrets = new OAuthSecrets().consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
    WebResource wr = client.resource(hostname);
    wr.addFilter(filter);

    ClientResponse clientResponse = wr.path("requestToken").post(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      Form responseParameters = clientResponse.getEntity(Form.class);

      String token = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
      String tokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

      ExternalToken externalToken = new ExternalToken();
      externalToken.setNetwork(getName());
      externalToken.setType(ExternalToken.Type.REQUEST_TOKEN);
      externalToken.setCreationDate(System.currentTimeMillis());
      externalToken.setToken(token);
      externalToken.setSecret(tokenSecret);
      externalNetworkBean.createExternalToken(externalToken);

      try {
        return Response.seeOther(new URI(hostname + "/oauth/authenticate?" + OAuthParameters.TOKEN + "=" + URLEncoder.encode(token, "UTF-8"))).build();
      } catch (UnsupportedEncodingException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      } catch (URISyntaxException ex) {
        throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
      }
    } else {
      throw new WebApplicationException(clientResponse.getStatus());
    }
  }

  @Override
  public ExternalToken callback(String requestToken, String verifier) {
    ExternalToken externalRequestToken = externalNetworkBean.findExternalToken(getName(), ExternalToken.Type.REQUEST_TOKEN, requestToken);
    if (externalRequestToken != null) {
      WebResource wr = client.resource(hostname);
      OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
              consumerKey(consumerKey).version("1.0").token(externalRequestToken.getToken()).verifier(verifier);
      OAuthSecrets secrets = new OAuthSecrets().tokenSecret(externalRequestToken.getSecret()).consumerSecret(consumerSecret);
      OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
      wr.addFilter(filter);

      ClientResponse clientResponse = wr.path("accessToken").post(ClientResponse.class);
      if (clientResponse.getStatus() == 200) {
        Form responseParameters = clientResponse.getEntity(Form.class);

        String accessToken = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
        String accessTokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

        // delete request token
        externalNetworkBean.removeExternalToken(externalRequestToken.getId());

        ExternalToken externalAccessToken = new ExternalToken();
        externalAccessToken.setNetwork(getName());
        externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
        externalAccessToken.setCreationDate(System.currentTimeMillis());
        externalAccessToken.setToken(accessToken);
        externalAccessToken.setSecret(accessTokenSecret);
        externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

        return externalAccessToken;
      } else {
        throw new WebApplicationException(clientResponse.getStatus());
      }
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

  @Override
  public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
		throw new WebApplicationException(Response.status(Response.Status.BAD_REQUEST).entity("Not yet implemented.").build());
  }

	@Override
	public List<Stream> getStream(OnlineAccount account) {
		return new LinkedList<Stream>();
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
		return new LinkedList<Friend>();
	}
}
