package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.ejb.UserBean;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.ejb.DaliCoreSocialBean;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.Company;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.Job;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.Client;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

/**
 *
 * @author joeri
 */
public abstract class ExternalNetwork {
	public static final String DALICORESOCIAL_LOCALCALLBACK = "dalicoresocial_localcallback";
	public static final String DALICORESOCIAL_USER = "dalicoresocial_user";
	public static final String DALICORESOCIAL_UPDATEUSER = "dalicoresocial_updateuser";
  public static final String daliKey = "daliKey";

	protected static final Client client = Client.create();

	private String name;
  protected String consumerKey;
  protected String consumerSecret;

  protected DaliCoreSocialBean externalNetworkBean = lookup(DaliCoreSocialBean.class);
  protected UserBean userBean = lookup(UserBean.class);

  public ExternalNetwork(String consumerKey, String consumerSecret) {
		this(consumerKey, consumerSecret, null);
  }

  public ExternalNetwork(String consumerKey, String consumerSecret, String name) {
    this.consumerKey = consumerKey;
    this.consumerSecret = consumerSecret;
		if (name == null || name.trim().isEmpty()) {
			this.name = getClass().getName();
		} else {
			this.name = name;
		}
  }

  public String getName() {
    return name;
  }

  protected <T> T lookup(Class<T> clazz) {
    try {
      Context c = new InitialContext();
      return (T) c.lookup("java:module/" + clazz.getSimpleName());
    } catch (NamingException ne) {
      Constants.LOGGER.log(Level.SEVERE, "exception caught", ne);
      throw new RuntimeException(ne);
    }
  }

	public Response connect(HttpServletRequest request, UriInfo ui, String localCallback) {
		// set the current external network on the active session
		request.getSession().setAttribute(DALICORESOCIAL_LOCALCALLBACK, localCallback);

		String oauthCallback = ui.getBaseUri().getScheme() + "://" + ui.getBaseUri().getAuthority() + ui.getBaseUri().getPath() + "dalicoresocial/callback/" + getName();
		return connect(oauthCallback);
	}

  public abstract Response connect(String oauthCallback);

  public abstract ExternalToken callback(String requestToken, String verifier);

	public OnlineAccount registerOnlineAccount(String appKey, User user, ExternalToken externalToken, Boolean updateUser) throws DaliCoreException {
		OnlineAccount onlineAccount = createOnlineAccount(appKey, externalToken);
    onlineAccount.getUser().setAppKey(appKey);
		if (onlineAccount.getId() == null) {
			if (user == null) {
				user = onlineAccount.getUser();
				if (user.getId() == null) {
					user = userBean.create(user);
				}
			} else if (updateUser != null && updateUser) {
				user.setScreenName(onlineAccount.getUser().getScreenName());
				user.setFirstName(onlineAccount.getUser().getFirstName());
				user.setLastName(onlineAccount.getUser().getLastName());
				user.setDepiction(onlineAccount.getUser().getDepiction());
				user = userBean.update(user);
			}
			onlineAccount.setUser(user);
			onlineAccount.setAccessToken(externalToken);
			onlineAccount = externalNetworkBean.createOnlineAccount(onlineAccount);
		} else if (onlineAccount.getUser().getId() == null) {
			user = userBean.create(onlineAccount.getUser());
			onlineAccount.setUser(user);
			if (onlineAccount.getAccessToken() != null && onlineAccount.getAccessToken().getId() != externalToken.getId()) {
				externalNetworkBean.removeExternalToken(onlineAccount.getAccessToken().getId());
			}
			onlineAccount.setAccessToken(externalToken);
			onlineAccount = externalNetworkBean.updateOnlineAccount(onlineAccount);
		}
		return onlineAccount;
	}

	public abstract OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException;
	public abstract List<Stream> getStream(OnlineAccount account);
	public abstract List<Friend> getFriends(OnlineAccount account);

  public Collection<Company> searchCompanies(OnlineAccount account, String searchKey) {
    Set<Company> answer = new HashSet();
    return answer;
  }

  public Company getCompany(OnlineAccount account, String id) {
    return null;
  }
  
  public Collection<Job> searchJobsByCompany (OnlineAccount account, String companyName) {
    return new HashSet<Job>();
  }
  
  public Collection<Actor> searchPeople (OnlineAccount account, String keyword) {
    return new HashSet<Actor>();
  }
}
