package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;
import java.util.logging.Level;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

/**
 *
 * @author joeri
 */
public class FacebookExternalNetwork extends ExternalNetwork {
	private static DateFormat dfStatus = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'+0000'", Locale.US);

	public FacebookExternalNetwork(String consumerKey, String consumerSecret) {
		super(consumerKey, consumerSecret, "facebook");
	}

	public FacebookExternalNetwork(String consumerKey, String consumerSecret, String name) {
		super(consumerKey, consumerSecret, name);
	}

	@Override
	public Response connect(String callback) {
		try {
      return Response.seeOther(new URI("https://www.facebook.com/dialog/oauth?client_id=" + URLEncoder.encode(consumerKey, "UTF-8") + "&scope=publish_stream,email,read_stream&redirect_uri=" + URLEncoder.encode(callback, "UTF-8"))).build();
		} catch (UnsupportedEncodingException ex) {
			throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
		} catch (URISyntaxException ex) {
			throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
		}
	}

	@Override
	public ExternalToken callback(String token, String verifier) {
		WebResource wr = client.resource("https://graph.facebook.com");

		String encodedVerifier = verifier;
		try {
			encodedVerifier = URLEncoder.encode(verifier, "UTF-8");
		} catch (UnsupportedEncodingException ex) {
			Constants.LOGGER.log(Level.SEVERE, null, ex);
		}

		ClientResponse clientResponse = wr.path("oauth/access_token").
						queryParam("client_id", consumerKey).
						queryParam("client_secret", consumerSecret).
						queryParam("code", token).
						queryParam("redirect_uri", encodedVerifier).
						get(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			String response = clientResponse.getEntity(String.class);
			String accessToken;
			if (response.indexOf("expires") > 0) {
				accessToken = response.substring(response.indexOf("access_token=") + 13, response.indexOf("&expires="));
			} else {
				accessToken = response.substring(response.indexOf("access_token=") + 13);
			}

			if (accessToken != null) {
				ExternalToken externalAccessToken = new ExternalToken();
				externalAccessToken.setNetwork(getName());
				externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
				externalAccessToken.setCreationDate(System.currentTimeMillis());
				externalAccessToken.setToken(accessToken);
				externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

				return externalAccessToken;
			} else {
				throw new WebApplicationException(Response.Status.BAD_REQUEST);
			}
		} else {
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}

	@Override
	public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
		WebResource wr = client.resource("https://graph.facebook.com");
		ClientResponse clientResponse = wr.path("me")
						.queryParam("access_token", accessToken.getToken())
						.queryParam("fields", "id,first_name,last_name,picture,email")
						.get(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			String me = clientResponse.getEntity(String.class);

			String id = null, firstName = null, lastName = null, pictureUrl = null, email = null;
			try {
				JSONObject jsonObject = new JSONObject(me);

				id = jsonObject.getString("id");
				firstName = jsonObject.getString("first_name");
				lastName = jsonObject.getString("last_name");
				email = jsonObject.getString("email");
				JSONObject jsonPicture = jsonObject.optJSONObject("picture");
				if (jsonPicture != null) {
					JSONObject jsonPictureData = jsonPicture.optJSONObject("data");
					if (jsonPictureData != null) {
						pictureUrl = jsonPictureData.optString("url");
					}
				} else {
					pictureUrl = jsonObject.optString("picture");
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			OnlineAccount onlineAccount = externalNetworkBean.findOnlineAccount(appKey, getName(), id);
			if (onlineAccount == null) {
				onlineAccount = new OnlineAccount();
				onlineAccount.setAppKey(appKey);
				onlineAccount.setCreationDate(System.currentTimeMillis());
				onlineAccount.setNetwork(getName());
				onlineAccount.setAccountId(id);
			}

			if (onlineAccount.getUser() == null) {
				User user = new User();
				user.setEmail(email);
				user.setFirstName(firstName);
				user.setLastName(lastName);
				user.setScreenName(user.getDisplayName());
				user.setDepiction(pictureUrl);
				onlineAccount.setUser(user);
			}

			return onlineAccount;
		} else {
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}

	@Override
	public List<Stream> getStream(OnlineAccount account) {
		WebResource wr = client.resource("https://graph.facebook.com");
		ClientResponse clientResponse = wr.path("me/home")
						.queryParam("access_token", account.getAccessToken().getToken())
						.get(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			String home = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Facebook News: {0}", home);

			List<Stream> streams = new LinkedList<Stream>();

			try {
				JSONObject jsonHome = new JSONObject(home);
				JSONArray jsonData = jsonHome.getJSONArray("data");

				for (int i = 0; i < jsonData.length(); i++) {
					JSONObject jsonStream = jsonData.getJSONObject(i);
					if ("status".equals(jsonStream.getString("type"))) {
						Stream stream = new Stream();
						stream.setNetwork(getName());
						stream.setUid(UUID.randomUUID().toString());
						stream.setRemoteId(jsonStream.getString("id"));
						stream.setContent(jsonStream.optString("message"));

						try {
							String createdTime = jsonStream.getString("created_time");
							stream.setCreationDate(dfStatus.parse(createdTime).getTime());
						} catch (ParseException ex) {
							Constants.LOGGER.log(Level.SEVERE, null, ex);
							stream.setCreationDate(System.currentTimeMillis());
						}

						JSONObject from = jsonStream.optJSONObject("from");
						if (from != null) {
							Actor actor = new Actor();
							actor.setUid(UUID.randomUUID().toString());
							actor.setRemoteId(from.optString("id"));
							actor.setScreenName(from.optString("name"));
							actor.setDepiction("https://graph.facebook.com/" + from.optString("id") + "/picture");
							stream.setAuthor(actor);
						}

						streams.add(stream);
					}
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			return streams;
		} else {
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
		WebResource wr = client.resource("https://graph.facebook.com");
		ClientResponse clientResponse = wr.path("me/friends")
						.queryParam("access_token", account.getAccessToken().getToken())
						.queryParam("fields", "id,first_name,last_name,picture,username")
						.get(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			String meFriends = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Facebook Friends: {0}", meFriends);

			List<Friend> friends = new LinkedList<Friend>();

			try {
				JSONObject jsonMeFriends = new JSONObject(meFriends);
				JSONArray jsonData = jsonMeFriends.getJSONArray("data");

				for (int i = 0; i < jsonData.length(); i++) {
					JSONObject jsonUser = jsonData.getJSONObject(i);
					Friend friend = new Friend();
					friend.setNetwork(getName());
					friend.setUid(UUID.randomUUID().toString());
					friend.setRemoteId(jsonUser.optString("id"));
					friend.setScreenName(jsonUser.optString("username"));
					friend.setFirstName(jsonUser.optString("first_name"));
					friend.setLastName(jsonUser.optString("last_name"));
					JSONObject jsonPicture = jsonUser.optJSONObject("picture");
					if (jsonPicture != null) {
						JSONObject jsonPictureData = jsonPicture.optJSONObject("data");
						if (jsonPictureData != null) {
							friend.setDepiction(jsonPictureData.optString("url"));
						}
					} else {
						friend.setDepiction(jsonUser.optString("picture"));
					}
					friends.add(friend);
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}

			return friends;
		} else {
			Constants.LOGGER.log(Level.INFO, "Facebook response not 200: {0}", clientResponse.getEntity(String.class));
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}
}
