package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.lodgon.dali.core.social.util.DateParser;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.representation.Form;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

/**
 *
 * @author joeri
 */
public class GooglePlusExternalNetwork extends ExternalNetwork {
  public GooglePlusExternalNetwork(String consumerKey, String consumerSecret) {
    super(consumerKey, consumerSecret, "googleplus");
  }

  public GooglePlusExternalNetwork(String consumerKey, String consumerSecret, String name) {
    super(consumerKey, consumerSecret, name);
  }

  @Override
  public Response connect(String callback) {
		try {
			return Response.seeOther(new URI("https://accounts.google.com/o/oauth2/auth?client_id=" + URLEncoder.encode(consumerKey, "UTF-8") + "&scope=" + URLEncoder.encode("https://www.googleapis.com/auth/plus.me https://www.googleapis.com/auth/userinfo.email", "UTF-8") + "&response_type=code&redirect_uri=" + URLEncoder.encode(callback, "UTF-8"))).build();
		} catch (UnsupportedEncodingException ex) {
			throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
		} catch (URISyntaxException ex) {
			throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
		}
  }

  @Override
  public ExternalToken callback(String code, String verifier) {
		WebResource wr = client.resource("https://accounts.google.com");

		Form form = new Form();
		form.add("client_id", consumerKey);
		form.add("client_secret", consumerSecret);
		form.add("code", code);
		form.add("redirect_uri", verifier);
		form.add("grant_type", "authorization_code");

		ClientResponse clientResponse = wr.path("o/oauth2/token")
						.type(MediaType.APPLICATION_FORM_URLENCODED_TYPE)
						.post(ClientResponse.class, form);
		if (clientResponse.getStatus() == 200) {
			String response = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "OAuth2 Access Token response: {0}", response);

			try {
				JSONObject jsonResponse = new JSONObject(response);

				String accessToken = jsonResponse.optString("access_token");
				if (accessToken != null) {
					ExternalToken externalAccessToken = new ExternalToken();
					externalAccessToken.setNetwork(getName());
					externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
					externalAccessToken.setCreationDate(System.currentTimeMillis());
					externalAccessToken.setToken(accessToken);
					externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

					return externalAccessToken;
				} else {
					throw new WebApplicationException(Response.Status.BAD_REQUEST);
				}
			} catch (JSONException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
				throw new WebApplicationException(ex, Response.Status.BAD_REQUEST);
			}
    } else {
      throw new WebApplicationException(Response.ok(clientResponse.getEntity(String.class)).status(clientResponse.getStatus()).build());
    }
  }

  @Override
  public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
    WebResource wr = client.resource("https://www.googleapis.com");

    ClientResponse clientResponse = wr.path("/plus/v1/people/me")
						.queryParam("access_token", accessToken.getToken())
						.get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String json = clientResponse.getEntity(String.class);

      String id = "";
      String screenName = "";
      String firstName = "";
      String lastName = "";
      String pictureUrl = "";
      try {
        JSONObject jsonObject = new JSONObject(json);

        id = jsonObject.getString("id");
        screenName = jsonObject.getString("displayName");
        firstName = jsonObject.getJSONObject("name").getString("givenName");
        lastName = jsonObject.getJSONObject("name").getString("familyName");
        pictureUrl = jsonObject.getJSONObject("image").getString("url");
      } catch (JSONException ex) {
        Constants.LOGGER.log(Level.SEVERE, null, ex);
      }

      OnlineAccount onlineAccount = externalNetworkBean.findOnlineAccount(appKey, getName(), id);
      if (onlineAccount == null) {
        onlineAccount = new OnlineAccount();
				onlineAccount.setAppKey(appKey);
        onlineAccount.setCreationDate(System.currentTimeMillis());
        onlineAccount.setNetwork(getName());
        onlineAccount.setAccountId(id);
      }

      if (onlineAccount.getUser() == null) {
        User user = new User();
        user.setScreenName(screenName);
        user.setFirstName(firstName);
        user.setLastName(lastName);
        user.setDepiction(pictureUrl);
        onlineAccount.setUser(user);
      }

      return onlineAccount;
    } else {
      throw new WebApplicationException(Response.Status.BAD_REQUEST);
    }
  }

	@Override
	public List<Stream> getStream(OnlineAccount account) {
    WebResource wr = client.resource("https://www.googleapis.com");

    ClientResponse clientResponse = wr.path("/plus/v1/people/me/activities/public")
						.queryParam("access_token", account.getAccessToken().getToken())
						.get(ClientResponse.class);
    if (clientResponse.getStatus() == 200) {
      String activities = clientResponse.getEntity(String.class);

			Constants.LOGGER.log(Level.FINER, "Google+ Activities: {0}", activities);

			List<Stream> streams = new LinkedList<Stream>();

			try {
        JSONObject jsonActivities = new JSONObject(activities);
				JSONArray jsonItems = jsonActivities.getJSONArray("items");
				for (int i = 0; i < jsonItems.length(); i++) {
					JSONObject jsonActivity = jsonItems.getJSONObject(i);
					Stream stream = new Stream();
					stream.setNetwork(getName());
					stream.setUid(UUID.randomUUID().toString());
					stream.setRemoteId(jsonActivity.getString("id"));
					stream.setUri(jsonActivity.getString("url"));
					stream.setTitle(jsonActivity.getString("title"));
					stream.setContent(jsonActivity.getJSONObject("object").getString("originalContent"));

					String published = jsonActivity.getString("published");
					stream.setCreationDate(DateParser.parseDate(published).getTime());

					JSONObject jsonActor = jsonActivity.optJSONObject("actor");
					if (jsonActor != null) {
						Actor actor = new Actor();
						actor.setUid(UUID.randomUUID().toString());
						actor.setRemoteId(jsonActor.optString("id"));
						actor.setScreenName(jsonActor.optString("displayName"));
						JSONObject jsonImage = jsonActor.optJSONObject("image");
						if (jsonImage != null) {
							actor.setDepiction(jsonImage.optString("url"));
						}
						stream.setAuthor(actor);
					}

					streams.add(stream);
				}
      } catch (JSONException ex) {
        Constants.LOGGER.log(Level.SEVERE, null, ex);
      }

			return streams;
    } else {
      throw new WebApplicationException(Response.ok(clientResponse.getEntity(String.class)).status(clientResponse.getStatus()).build());
    }
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
		return new LinkedList<Friend>();
	}
}
