package com.lodgon.dali.core.social;

import com.lodgon.dali.core.ejb.DaliCoreException;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.social.entity.Actor;
import com.lodgon.dali.core.social.entity.Company;
import com.lodgon.dali.core.social.entity.ExternalToken;
import com.lodgon.dali.core.social.entity.Friend;
import com.lodgon.dali.core.social.entity.Job;
import com.lodgon.dali.core.social.entity.OnlineAccount;
import com.lodgon.dali.core.social.entity.Stream;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.filter.LoggingFilter;
import com.sun.jersey.api.representation.Form;
import com.sun.jersey.oauth.client.OAuthClientFilter;
import com.sun.jersey.oauth.signature.HMAC_SHA1;
import com.sun.jersey.oauth.signature.OAuthParameters;
import com.sun.jersey.oauth.signature.OAuthSecrets;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @author erwin
 */
public class LinkedinExternalNetwork extends ExternalNetwork {

	public LinkedinExternalNetwork(String consumerKey, String consumerSecret) {
		super(consumerKey, consumerSecret, "linkedin");
	}

	public LinkedinExternalNetwork(String consumerKey, String consumerSecret, String name) {
		super(consumerKey, consumerSecret, name);
	}

	@Override
	public Response connect(String callback) {
		OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
						consumerKey(consumerKey).version("1.0").callback(callback);
		OAuthSecrets secrets = new OAuthSecrets().consumerSecret(consumerSecret);
		OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
		WebResource wr = client.resource("https://api.linkedin.com");
		wr.addFilter(filter);

		ClientResponse clientResponse = wr.path("/uas/oauth/requestToken").type(MediaType.APPLICATION_FORM_URLENCODED).post(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			Form responseParameters = clientResponse.getEntity(Form.class);

			String url = (String) responseParameters.getFirst("xoauth_request_auth_url");
			String token = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
			String tokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

			ExternalToken externalToken = new ExternalToken();
			externalToken.setNetwork(getName());
			externalToken.setType(ExternalToken.Type.REQUEST_TOKEN);
			externalToken.setCreationDate(System.currentTimeMillis());
			externalToken.setToken(token);
			externalToken.setSecret(tokenSecret);
			externalNetworkBean.createExternalToken(externalToken);

			try {
				return Response.seeOther(new URI(url + "?" + OAuthParameters.TOKEN + "=" + URLEncoder.encode(token, "UTF-8"))).build();
			} catch (UnsupportedEncodingException ex) {
				throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
			} catch (URISyntaxException ex) {
				throw new WebApplicationException(ex, Response.Status.INTERNAL_SERVER_ERROR);
			}
		} else {
			Constants.LOGGER.log(Level.INFO, "Twitter did not allow us to connect, statuscode = {0}", clientResponse.getStatus());
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}

	}

	@Override
	public ExternalToken callback(String requestToken, String verifier) {
		ExternalToken externalRequestToken = externalNetworkBean.findExternalToken(getName(), ExternalToken.Type.REQUEST_TOKEN, requestToken);
		if (externalRequestToken != null) {
			WebResource wr = client.resource("https://api.linkedin.com");
			OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
							consumerKey(consumerKey).version("1.0").token(externalRequestToken.getToken()).verifier(verifier);
			OAuthSecrets secrets = new OAuthSecrets().tokenSecret(externalRequestToken.getSecret()).consumerSecret(consumerSecret);
			OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
			wr.addFilter(filter);

			ClientResponse clientResponse = wr.path("/uas/oauth/accessToken").get(ClientResponse.class);
			if (clientResponse.getStatus() == 200) {
				Form responseParameters = clientResponse.getEntity(Form.class);

				String accessToken = (String) responseParameters.getFirst(OAuthParameters.TOKEN);
				String accessTokenSecret = (String) responseParameters.getFirst(OAuthParameters.TOKEN_SECRET);

				// delete request token
				externalNetworkBean.removeExternalToken(externalRequestToken.getId());

				ExternalToken externalAccessToken = new ExternalToken();
				externalAccessToken.setNetwork(getName());
				externalAccessToken.setType(ExternalToken.Type.ACCESS_TOKEN);
				externalAccessToken.setCreationDate(System.currentTimeMillis());
				externalAccessToken.setToken(accessToken);
				externalAccessToken.setSecret(accessTokenSecret);
				externalAccessToken = externalNetworkBean.createExternalToken(externalAccessToken);

				return externalAccessToken;
			} else {
				throw new WebApplicationException(Response.Status.BAD_REQUEST);
			}
		} else {
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}

	@Override
	public OnlineAccount createOnlineAccount(String appKey, ExternalToken accessToken) throws DaliCoreException {
		WebResource wr = client.resource("https://api.linkedin.com");
		OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
						consumerKey(consumerKey).version("1.0").token(accessToken.getToken());
		OAuthSecrets secrets = new OAuthSecrets().tokenSecret(accessToken.getSecret()).consumerSecret(consumerSecret);
		OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);
		wr.addFilter(filter);

		ClientResponse clientResponse = wr.path("/v1/people/~:(id,first-name,last-name,email-address,picture-url)").get(ClientResponse.class);
		if (clientResponse.getStatus() == 200) {
			String xml = clientResponse.getEntity(String.class);

			String id = "";
			String firstName = "";
			String lastName = "";
			String pictureUrl = "";
			String email = "";

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			try {
				InputSource source = new InputSource(new StringReader(xml));
				Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(source);

				XPathExpression idExpression = xpath.compile("/person/id");
				XPathExpression firstNameExpression = xpath.compile("/person/first-name");
				XPathExpression lastNameExpression = xpath.compile("/person/last-name");
				XPathExpression emailExpression = xpath.compile("/person/email-address");
				XPathExpression pictureUrlExpression = xpath.compile("/person/picture-url");

				id = idExpression.evaluate(document);
				firstName = firstNameExpression.evaluate(document);
				lastName = lastNameExpression.evaluate(document);
				pictureUrl = pictureUrlExpression.evaluate(document);
				email = emailExpression.evaluate(document);
			} catch (ParserConfigurationException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			} catch (IOException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			} catch (SAXException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			} catch (XPathExpressionException ex) {
				Constants.LOGGER.log(Level.SEVERE, null, ex);
			}


			OnlineAccount onlineAccount = externalNetworkBean.findOnlineAccount(appKey, getName(), id);
			if (onlineAccount == null) {
				onlineAccount = new OnlineAccount();
				onlineAccount.setAppKey(appKey);
				onlineAccount.setCreationDate(System.currentTimeMillis());
				onlineAccount.setNetwork(getName());
				onlineAccount.setAccountId(id);
			}

			if (onlineAccount.getUser() == null) {
				User user = new User();
				user.setEmail(email);
				user.setFirstName(firstName);
				user.setLastName(lastName);
				user.setScreenName(user.getDisplayName());
				user.setDepiction(pictureUrl);
				onlineAccount.setUser(user);
			}


			return onlineAccount;
		} else {
			throw new WebApplicationException(Response.Status.BAD_REQUEST);
		}
	}

	@Override
	public List<Stream> getStream(OnlineAccount account) {
		return new LinkedList();//TODO
	}

	@Override
	public List<Friend> getFriends(OnlineAccount account) {
		return new LinkedList();//TODO
	}
    
    @Override
    public Collection<Company> searchCompanies(OnlineAccount account, String searchKey) {
        WebResource wr = client.resource("http://api.linkedin.com");
        OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
                consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
        OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
        OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

        wr.addFilter(new LoggingFilter());
        wr.addFilter(filter);
        ClientResponse clientResponse = wr.path("v1/company-search")
                .queryParam("keywords", searchKey)
                .get(ClientResponse.class);
        SearchCompanyResults res = clientResponse.getEntity(SearchCompanyResults.class);
        List<Company> answer = res.getCompany();
        return answer;
    }
    
    @Override
    public Company getCompany (OnlineAccount account, String cid) {
        WebResource wr = client.resource("http://api.linkedin.com");
        OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
                consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
        OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
        OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

        wr.addFilter(new LoggingFilter());
        wr.addFilter(filter);
        ClientResponse clientResponse = wr.path("v1/companies").path(cid+":(id,name,website-url,logo-url,description)")
                .get(ClientResponse.class);
       Company answer = clientResponse.getEntity(Company.class);
       return answer;
    }

    @Override
    public Collection<Job> searchJobsByCompany (OnlineAccount account, String companyName) {
     WebResource wr = client.resource("http://api.linkedin.com");
        OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
                consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
        OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
        OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

        wr.addFilter(new LoggingFilter());
        wr.addFilter(filter);
        ClientResponse clientResponse = wr.path("v1/job-search")
                .queryParam("company-name", companyName)
                .get(ClientResponse.class);
       SearchJobResults raw = clientResponse.getEntity(SearchJobResults.class);
       List<Job> rawJobs = raw.getJob();
       List<Job> answer = new LinkedList<Job>();
       for (Job candidate: rawJobs) {
         answer.add(getDetailedJob(account, candidate.getId()));
       }
       return answer;
    }
    
    private Job getDetailedJob (OnlineAccount account, String jobId) {
           WebResource wr = client.resource("http://api.linkedin.com");

       OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
                consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
        OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
        OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

        wr.addFilter(new LoggingFilter());
        wr.addFilter(filter);
        ClientResponse clientResponse = wr.path("v1/jobs")
                .path(jobId+":(id,position:(title),description,description-snippet)")
                .get(ClientResponse.class);
        Job answer = clientResponse.getEntity(Job.class);
        return answer;
    }
    
  @Override
  public Collection<Actor> searchPeople (OnlineAccount account, String keyword) {
    WebResource wr = client.resource("http://api.linkedin.com");

    OAuthParameters params = new OAuthParameters().signatureMethod(HMAC_SHA1.NAME).
            consumerKey(consumerKey).version("1.0").token(account.getAccessToken().getToken());
    OAuthSecrets secrets = new OAuthSecrets().tokenSecret(account.getAccessToken().getSecret()).consumerSecret(consumerSecret);
    OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), params, secrets);

    wr.addFilter(new LoggingFilter());
    wr.addFilter(filter);
    ClientResponse clientResponse = wr.path("v1/people-search")
            .queryParam("keywords", keyword).get(ClientResponse.class);
    int status = clientResponse.getStatus();
    Set<Actor> answer = new HashSet();
    if (status != 200) {
			Constants.LOGGER.log(Level.SEVERE, "Searching people on linkedin returned HTTP response {0}", status);
      String msg = clientResponse.getEntity(String.class);
      Constants.LOGGER.log(Level.INFO, "Detailed error message: {0}", msg);
      return answer;
    }
    SearchPeopleResults results = clientResponse.getEntity(SearchPeopleResults.class);
    List<Person> people = results.getPerson();

    for (Person person : people) {
      answer.add(person.toActor());
    }
    return answer;
}
  
  @XmlRootElement(name="people-search")
  @XmlAccessorType(XmlAccessType.FIELD)
  static class SearchPeopleResults {
    @XmlElementWrapper(name="people")
    private List<Person> person;
    public SearchPeopleResults() {}

    /**
     * @return the person
     */
    public List<Person> getPerson() {
      return person;
    }

    /**
     * @param person the person to set
     */
    public void setPerson(List<Person> person) {
      this.person = person;
    }
    
  }
  
  @XmlRootElement(name="person")
  @XmlAccessorType(XmlAccessType.FIELD)
  static class Person {
    @XmlElement(name="first-name")
    private String firstName;
    @XmlElement(name="last-name")
    private String lastName;
    @XmlElement(name="id")
    private String remoteId;
    public Person() {}
    public Actor toActor() {
      Actor answer = new Actor();
      answer.setFirstName(getFirstName());
      answer.setLastName(getLastName());
      answer.setRemoteId(getRemoteId());
      return answer;
    }

    /**
     * @return the firstName
     */
    public String getFirstName() {
      return firstName;
    }

    /**
     * @param firstName the firstName to set
     */
    public void setFirstName(String firstName) {
      this.firstName = firstName;
    }

    /**
     * @return the lastName
     */
    public String getLastName() {
      return lastName;
    }

    /**
     * @param lastName the lastName to set
     */
    public void setLastName(String lastName) {
      this.lastName = lastName;
    }

    /**
     * @return the remoteId
     */
    public String getRemoteId() {
      return remoteId;
    }

    /**
     * @param remoteId the remoteId to set
     */
    public void setRemoteId(String remoteId) {
      this.remoteId = remoteId;
    }
  }
    @XmlRootElement(name="job-search") 
    static class SearchJobResults {
      public SearchJobResults() {}
      private List<Job> job;
      
      @XmlElementWrapper(name="jobs")
      public List<Job> getJob() {
        return job;
      }
      
      public void setJob(List<Job> job) {
        this.job = job;
      }
    }
    
    @XmlRootElement(name="company-search")
    static class SearchCompanyResults {
        public SearchCompanyResults() {}
        private List<Company> company;

        /**
         * @return the companies
         */
        @XmlElementWrapper(name="companies")
        public List<Company> getCompany() {
            return company;
        }

        /**
         * @param companies the companies to set
         */
        public void setCompany(List<Company> company) {
            this.company = company;
        }
    }
}
